import asyncio, hashlib, json, os, sys, validators

from IzzyLib import logging
from IzzyLib.misc import DotDict, DefaultDict
from urllib.parse import urlparse
from json.decoder import *
from sanic.views import HTTPMethodView

from . import messages
from .misc import sanitize

from ..config import config
from ..database import db, Row
from ..functions import ApDate, Error, JsonResp


class Post(object):
	def get(self, key):
		return getattr(self, key, None)


	def apps_none(self, request, data, args):
		data = DotDict(data)
		fields = ['redirect_uris', 'scopes', 'client_name', 'website']

		for line in fields:
			if line not in data:
				data[line] = None

		if None in [data.scopes, data.client_name]:
			logging.debug('Missing scope or name for app')
			logging.debug(f'scope: {data.scope}, name: {data.client_name}')
			return 'MissingData'

		#scopes = scope_check(data.scopes)

		if data.scopes == None:
			logging.debug(f'Invalid scopes: {data.scope}')
			return 'InvalidScope'

		#if not validators.url(data.redirect_uris):
			#logging.debug(f'Invalid app URL: {data.redirect_uris}')
			#return 'InvalidURL'

		with db.session() as s:
			row = s.put.app(data.redirect_uris, data.scopes, data.client_name, data.website)

		return {'client_id': row.client_id, 'client_secret': row.client_secret, 'redirect_uri': row.redirect_uri, 'scopes': row.scope}


	def statuses_none(self, request, data, args):
		data = DefaultDict(data)
		user = request.ctx.token_user

		if not any([data.status, data.spoiler_text]):
			return JsonResp('Missing status text or warning', 400)

		with db.session() as s:
			s.insert('status',
				userid = user.id,
				domainid = user.domainid,
				hash = hashlib.sha256(data.status.encode("utf-8")).hexdigest(),
				content = data.status,
				warning = data.spoiler_text,
				visibility = data.visibility,
				mentions = None,
				replies = None
			)

		return {}


class Get(object):
	def get(self, key):
		return getattr(self, key, None)


	def accounts_verify_credentials(self, request, data, args):
		return messages.mastodon_user(request.ctx.token_user)


	def custom_emojis_none(self, request, data, args):
		emojis = []

		with db.session() as s:
			for row in s.fetch('emoji', domainid=config.domainid, enabled=True, single=False):
				url = f'https://{config.domain}/media/emojis/{row.domainid}/{row.shortcode[0]}/{row.shortcode}.png'

				emojis.append({
					'shortcode': row.shortcode,
					'url': url,
					'static_url': url,
					'visible_in_picker': row.display
				})

		return emojis


	def instance_none(self, request, data, args):
		#stats = db.get.server_stats()
		stats = {}

		with db.session() as s:
			settings = s.get.configs()

			data = {
				'uri': config.web_domain,
				'title': settings.name,
				'short_description': settings.description,
				'description': settings.get('short_description', settings.description),
				'email': None,
				'version': f'2.7.0 (compatible; BarksharkSocial {config.version})',
				'urls': {
					'streaming_api': 'wss://'+config.web_domain
				},
				'stats': {
					'user_count': s.get.user_count(),
					"status_count": s.get.status_count(),
					"domain_count": s.get.domain_count()
				},
				'thumbnail': None,
				'max_toot_chars': settings.char_limit,
				"poll_limits": {
					"max_options": 5,
					"max_option_chars": 100,
					"min_expiration": 300,
					"max_expiration": 2629746
				},
				"languages": [
					"en"
				],
				"registrations": False,
				"approval_required": False,
				"invites_enabled": False,
				"contact_account": None
			}

			return data


	def timelines_home(self, request, data, args):
		limit = data.get('limit', 20)
		local = data.get('local', False)
		max_id = data.get('max_id')
		min_id = data.get('min_id')

		user = request.ctx.token_user
		table = db.table.status

		msg = []

		with db.session() as s:
			row_query = s.query(table).filter_by(userid=user.id).order_by(table.id.desc()).limit(limit)
			rows = [Row(row, db) for row in row_query.all()]

			for row in rows:
				user = s.fetch('user', id=row.userid)
				msg.append(messages.mastodon_post(row, user))

		return msg
