import json

from ..config import config
from ..database import db
from ..functions import ApDate


def mastodon_user(user):
	with db.session() as s:
		last_status = s.query(s.table.status).filter_by(userid=user.id).order_by(s.table.status.id.desc()).limit(1).one_or_none()
		prefs = user.config if user.config else {}
		data = {
			'id': user.id,
			'username': user.handle,
			'acct': user.handle,
			'display_name': user.name,
			'locked': prefs.get('private', False),
			'bot': prefs.get('bot', False),
			'created_at': ApDate(user.timestamp),
			'note': user.bio,
			'url': f'https://{config.web_domain}/@{user.handle}',
			'avatar': f'https://{config.web_domain}/{user.avatar}',
			'avatar_static': f'https://{config.web_domain}/{user.avatar}',
			'header': f'https://{config.web_domain}/{user.header}',
			'header_static': f'https://{config.web_domain}/{user.header}',
			'followers_count': 0,
			'following_count': 0,
			'statuses_count': s.count('status', userid=user.id),
			'fields': [{'name': k, 'value': v, 'verified_at': None} for k,v in prefs.info_table.items()],
			'last_status_at': ApDate(last_status.timestamp)
		}

		if user.table:
			for k,v in user.table.items():
				data['fields'].append({'name': k, 'value': v, 'verified_at': None})

		return data


def mastodon_post(post, user):
	data = {
		'id': post.id,
		'uri': f'https://{config.web_domain}/status/{post.id}',
		'url': f'https://{config.web_domain}/:{post.id}',
		'account': mastodon_user(user),# User dict for the account which posted the status
		'in_reply_to_id': None,
		'in_reply_to_account_id': None,
		'reblog': False,# Denotes whether the toot is a reblog. If so, set to the original toot dict.
		'content': post.content,
		'created_at': '',# Creation time
		'reblogs_count': 0,# Number of reblogs
		'favourites_count': 0,# Number of favourites
		'reblogged': False,# Denotes whether the logged in user has boosted this toot
		'favourited': False,# Denotes whether the logged in user has favourited this toot
		'sensitive': False,# Denotes whether media attachments to the toot are marked sensitive
		'spoiler_text': post.warning,
		'visibility': post.visibility,# Toot visibility ('public', 'unlisted', 'private', or 'direct')
		'mentions': [],# A list of users dicts mentioned in the toot, as Mention dicts
		'media_attachments': [],# A list of media dicts of attached files
		'emojis': [],# A list of custom emojis used in the toot, as Emoji dicts
		'tags': [],# A list of hashtag used in the toot, as Hashtag dicts
		'bookmarked': False,# True if the status is bookmarked by the logged in user, False if not.
		'application': None,# Application dict for the client used to post the toot
		'language': 'en',# The language of the toot, if specified by the server, as ISO 639-1 (two-letter) language code.
		'muted': False,# Boolean denoting whether the user has muted this status by way of conversation muting
		'pinned': False,# Boolean denoting whether or not the status is currently pinned for the associated account.
		'replies_count': 0,# The number of replies to this status.
		'card': None,# A preview card for links from the status, if present at time of delivery, as card dict.
		'poll': None# A poll dict if a poll is attached to this status.
	}

	return data


#with db.session() as s:
	#user = s.get.user('izalia', config.domain)
	#print(json.dumps(mastodon_user(user), indent=4))
