import asyncio, json, os, sys, validators

from IzzyLib import logging
from IzzyLib.misc import RandomGen
#from json.decoder import * 
from sanic import response
from sanic.views import HTTPMethodView
from sanic.exceptions import Unauthorized, Forbidden
from sanic.response import json as rjson
from urllib.parse import urlparse

from .misc import sanitize

from ..config import config
from ..database import db
from ..functions import Error, JsonResp


def scope_check(scopes):
	read_write = ['follows', 'accounts', 'lists', 'blocks', 'mutes', 'bookmarks', 'notifications', 'favourites', 'search', 'filters', 'statuses']
	admin = ['read', 'write']
	admin_secc = ['accounts', 'reports']
	new_scopes = []

	for line in scopes:
		scope = line.split(':')

		if len(scope) < 2:
			new_scopes.append(line)
			continue

		if (scope[0] in ['read', 'write'] and scope[1] in read_write) or scope[0] in ['follow', 'push'] or (scope[0] == 'admin' and scope[1] in admin and scope[2]):
			new_scopes.append(line)

		else:
			logging.warning(f'Invalid scope: {line}')

	if len(new_scopes) < 1:
		return

	else:
		return new_scopes


class create:
	def app(redirect_uri, scope, name, url):
		if None in [scope, name]:
			logging.debug('Missing scope or name for app')
			logging.debug(f'scope: {scope}, name: {name}')
			return 'MissingData'

		#scopes = scope_check(scope)
		scopes = scope

		if scopes == None:
			logging.debug(f'Invalid scopes: {scope}')
			return 'InvalidScope'

		if not validators.url(url):
			logging.debug(f'Invalid app URL: {url}')
			return 'InvalidURL'

		with db.session() as s:
			row = s.put.app(redirect_uri, scopes, name, url)

		return {'client_id': row.client_id, 'client_secret': row.client_secret, 'redirect_uris': redirect_uri, 'scopes': scope}


	def authorize(app, userid):
		auth_code = RandomGen(40)

		with db.session() as s:
			if not s.fetch('user', id=userid):
				logging.error(f'social.api.oauth.create.authorize: Cannot find user with userid: {userid}')
				return

			s.insert('token',
				userid = userid,
				appid = app.id,
				code = auth_code
			)

			return auth_code


class Get:
	def authorize(request, data):
		login_token = request.cookies.get('login_token')
		client_id = data.get('client_id')
		redirect_uri = data.get('redirect_uri')

		if None in [client_id, redirect_uri, login_token]:
			raise InvalidUsage('Missing client_id or redirect_uri')

		app = db.fetch('app', client_id=client_id)

		if not app:
			return Error(request, 401, 'Invalid app')

		if data.get('accept') == None:
			return config.template.response('pages/oauth/authenticate.haml', request, {'args': data, 'app': app})

		if data.get('accept') == 'Reject':
			app.delete()

			return Error(request, 401, 'App denied')

		if not request.ctx.cookie:
			return Error(request, 401, 'Invalid login cookie')

		if data.get('response_type') == 'code':
			auth_code = create.authorize(app, request.ctx.cookie.userid)

		if not auth_code:
			return Error(request, 401, f'Failed to create auth code')

		if data.get('redirect_uri') == 'urn:ietf:wg:oauth:2.0:oob':
			return config.template.response('pages/oauth/authenticate.haml', request, {'app': app, 'code': auth_code})

		else:
			return response.redirect(f'{redirect_uri}?code={auth_code}')


	def token(request, data):
		if not data.get('code'):
			logging.debug(f'oauth.api.get.token: Missing code')
			return JsonResp({'error': 'missing_code'}, status=400)

		with db.session() as s:
			token = s.fetch('token', code=data['code'])

			if not token:
				logging.debug(f'oauth.api.get.token: Missing token')
				return JsonResp({'error': 'invalid_code'}, status=401)

			app = s.fetch('app', id=token.appid)

			if not app:
				logging.debug(f'oauth.api.get.token: Invalid app')
				token.delete_session()
				return JsonResp({'error': 'invalid_app'}, status=401)

			print('update token')
			token.update_session(s, **{'code': None, 'token': RandomGen(40)})

		return {
			'access_token': token.token, 
			'token_type': 'Bearer', 
			'scope': app.scope, 
			'created_at': int(app.timestamp.timestamp())
		}


class Post:
	token = Get.token


class InvalidUsage(Exception):
	pass
