from IzzyLib import logging
from datetime import datetime

from .schema import table, Base
from ..config import config


def setup(db, settings):
	Base.metadata.create_all(db.db)

	with db.session() as s:
		name = settings.get('name', 'Barkshark Social')
		s.put.version(config.dbversion)
		s.put.config('name', name)
		s.put.config('domain', settings.get('domain', 'example.com'))
		s.put.config('host', settings.get('example.com', 'example.com'))
		s.put.config('description', settings.get('description', 'Barkshark Social is an upcoming Activity Pub server with a focus on fredom of association, low resource usage, and cusomizability.'))
		s.put.config('char_limit', settings.get('char_limit', 5000), 'int')
		s.put.config('bio_limit', settings.get('bio_limit', 1000), 'int')
		s.put.config('secure', settings.get('secure', True), 'bool')

		domain = s.fetch('domain', domain=config.domain)

		if not domain:
			s.insert('domain',
				domain = config.domain,
				name = None,
				web_domain = config.web_domain,
				description = None,
				timestamp = datetime.now()
			)
			s.commit()

			domain = s.fetch('domain', domain=config.domain)

		else:
			domain.update_session(s, {'domain': config.domain, 'web_domain': config.web_domain})

		s.put.user('system', 
			name = name,
			bio = 'System account',
			permissions = 0,
			actor = f'https://{config.web_domain}/actor/system',
			inbox = f'https://{config.web_domain}/actor/system'
		)

		s.put.user('relay',
			name = f'{name} Relay',
			bio = 'System relay account',
			permissions = 0,
			actor = f'https://{config.web_domain}/actor/relay',
			inbox = f'https://{config.web_domain}/actor/relay'
		)

		s.put.user('relay',
			name = f'{name} Local Relay',
			bio = 'System local relay account',
			permissions = 0,
			actor = f'https://{config.web_domain}/actor/local',
			inbox = f'https://{config.web_domain}/actor/local'
		)


	print('Setup database :3')


def upgrade(dbversion, db):
	if dbversion == 0:
		setup(db)
		return

	logging.info('Migrated database to version', config.dbversion)
