import argparse, json, magic, os, shutil, sys, tarfile, zipfile
from os import environ as env
#from subprocess import Popen


from IzzyLib import logging
from IzzyLib.misc import RandomGen, Input, Path, Boolean, DotDict
from collections import Counter
from datetime import datetime

from . import messages
from .config import config
from .database import DataBase, db, migrate, schema
from .functions import Client, mkhash


parser = argparse.ArgumentParser()
parser.add_argument('command', help='Management command to run')
parser.add_argument('options', nargs=argparse.REMAINDER, help='Options passed to the management command')
pargs = parser.parse_args()
command, args = pargs.command, pargs.options


def man_test():
	for user in db.fetch('user', single=False):
		actor = 'https://social.barkshark.xyz/actor/'+user.handle
		user.update({
			'actor': actor,
			'inbox': actor
		})


def man_pushtest(objectid):
	with db.session() as s:
		status = s.fetch('status', id=int(objectid))
		user = status.user

	keyid = f'https://{config.web_domain}/actor/{user.handle}#main-key'
	headers = {'Accept': 'application/activity+json'}

	resp = Client.signed_request(user.privkey, keyid, 'https://barkshark.xyz/inbox',
		headers = headers,
		method = 'POST',
		data = messages.Announce(messages.Note(status))
	)

	return resp.text()


def man_httptest(url):
	with db.session() as s:
		user = s.fetch('user', handle='izalia')

	keyid = f'https://{config.web_domain}/actor/izalia#main-key'
	headers = {'Accept': 'application/activity+json'}

	args = [
		user.privkey,
		keyid,
		url
		##'https://barkshark.xyz/users/izalia/statuses/105572315304601364/activity'
	]

	kwargs = {
		'headers': headers,
	}

	resp = Client.signed_request(*args, **kwargs)

	try:
		return resp.json_pretty()
	except Exception as e:
		print(f'{e.__class__name}: {e}')

	return resp.text()


def man_allstar():
	text = '''Somebody once told me the world is gonna roll me
I ain't the sharpest tool in the shed
She was looking kind of dumb with her finger and her thumb
In the shape of an "L" on her forehead
Well, the years start coming and they don't stop coming
Fed to the rules and I hit the ground running
Didn't make sense not to live for fun
Your brain gets smart but your head gets dumb
So much to do, so much to see
So what's wrong with taking the back streets?
You'll never know if you don't go
You'll never shine if you don't glow
Hey, now, you're an all-star, get your game on, go play
Hey, now, you're a rock star, get the show on, get paid
And all that glitters is gold
Only shooting stars break the mold
It's a cool place and they say it gets colder
You're bundled up now wait 'til you get older
But the meteor men beg to differ
Judging by the hole in the satellite picture
The ice we skate is getting pretty thin
The water's getting warm so you might as well swim
My world's on fire. How about yours?
That's the way I like it and I'll never get bored
Hey, now, you're an all-star, get your game on, go play
Hey, now, you're a rock star, get the show on, get paid
And all that glitters is gold
Only shooting stars break the mold
Go for the moon
Go for the moon
Go for the moon
Go for the moon
Hey, now, you're an all-star, get your game on, go play
Hey, now, you're a rock star, get the show on, get paid
And all that glitters is gold
Only shooting stars
Somebody once asked could I spare some change for gas
I need to get myself away from this place
I said yep, what a concept
I could use a little fuel myself
And we could all use a little change
Well, the years start coming and they don't stop coming
Fed to the rules and I hit the ground running
Didn't make sense not to live for fun
Your brain gets smart but your head gets dumb
So much to do, so much to see
So what's wrong with taking the back streets?
You'll never know if you don't go
You'll never shine if you don't glow
Hey, now, you're an all star, get your game on, go play
Hey, now, you're a rock star, get the show on, get paid
And all that glitters is gold
Only shooting stars break the mold
And all that glitters is gold
Only shooting stars break the mold'''

	lines = text.splitlines()
	lines.reverse()
	with db.session() as s:
		for index, line in enumerate(lines):
			domainid = s.get.domainid(config.domain)
			user = s.get.user('izalia')
			time = datetime.now()

			if not user:
				return f'Invalid user: {name}'

			s.insert('status',
				userid = user.id,
				domainid = domainid,
				content = line,
				warning = f'Smash Mouth - Allstar {index+1}-{len(lines)}',
				timestamp = time
			)

	return 'You\'ve been Allstar-ed!'


def man_post(name, text, warning=None):
	with db.session() as s:
		domainid = s.get.domainid(config.domain)
		user = s.get.user(name)
		time = datetime.now()

		if not user:
			return f'Invalid user: {name}'

		s.insert('status',
			userid = user.id,
			domainid = domainid,
			content = text,
			warning = warning,
			timestamp = time
		)
		s.commit()

		return True if s.fetch('status', timestamp=time) else False


def man_sysacct(name=None):
	if not name:
		return 'Missing account name'

	if ' ' in name or not name.isalnum():
		return 'Invalid characters in name. User only alphanumeric characters'

	name = name.strip().lower()

	with db.session() as s:
		instname = s.get.config('name')
		s.put.user(name,
			name = f'{instname} {name.capitalize()}',
			bio = 'System account',
			permissions = 0
		)

		return s.get.user(name)


def man_setup():
	global db
	user = env.get('USER', 'social')
	env_exists = config.data.join(f'{config.env}.env').exists()

	if not env_exists or Input('Config file exists. Reconfigure?', False, Boolean, options=['true', 'false']):
		config.domain = Input('Domain name used for federation', config.domain)
		config.web_domain = Input('Domain used to access the instance from a web browser', config.web_domain)
		config.listen = Input('IP address for the server to listen on', config.listen)
		config.port = Input('Port number the server listens on', config.port, int)
		config.salt = Input('String to use when hashing a password', config.salt)
		config.secret = Input('String to user in a reverse proxy server to make sure it is valid', config.secret)

		config.dbtype = Input('The type of database to use for the backend', config.dbtype, options=['sqlite', 'postgresql'])

		if config.dbtype == 'sqlite':
			config.sq = DotDict()
			config.sqfile = Input('Name of the sqlite3 database file. Path is relative to the data directory', config.sqfile, Path)

		if config.dbtype == 'postgresql':
			config.db = DotDict()
			config.db.host = Input('Database host or socket location', config.db.host)
			config.db.port = Input('Database port', config.db.port, int)
			config.db.user = Input('Database user', config.db.user)
			config.db.password = Input('Database password', config.db.password)
			config.db.database = Input('Database name', config.db.database)
			config.db.maxconnections = Input('Max connections to the database', config.db.maxconnections, int)

		config.rd.host = Input('Redis host', config.rd.host)
		config.rd.port = Input('Redis port', config.rd.port, int)
		config.rd.user = Input('Redis user', config.rd.user)
		config.rd.password = Input('Redis password', config.rd.password, password=True)
		config.rd.database = Input('Redis database number', config.rd.database, int)
		config.rd.prefix = Input('Redis namespace prefix', config.rd.prefix)
		config.rd.maxconnections = Input('Max connections to Redis', config.rd.maxconnections, int)

		setup_env_file()

		dbconf = config.sq if config.dbtype == 'sqlite' else config.db
		db = DataBase(**dbconf, tables=schema.table)

	if Counter(db.table.keys()) != Counter(db.GetTables()):
		if Input('Database exists. Drop tables first?', False, Boolean, options=['true', 'false']):
			db.DropTables()

	settings = DotDict()

	if Input('Setup optional settings now? They can be changed later in the server settings', False, Boolean):
		settings.name = Input('Instance name', 'Barkshark Social')
		settings.description = Input('Instance description', 'UwU')
		settings.char_limit = Input('Character limit for posts', 5000, int)
		settings.bio_limit = Input('character limit for bios', 1000, int)
		settings.secure = Input('Should fetches from other instances require a signature?', True, Boolean)

		newset = [{'key': k, 'value': v, 'subtype': type(v).__name__} for k, v in settings.items()]
		db.put.configs(newset)

	migrate.setup(db, settings)

	if Input('Create an admin account now?', True, Boolean):
		man_adduser()
		logging.info('Created new admin account')

	return 'Instance all setup! Run it with "python3 -m social" :3'


def man_adduser(handle=None, email=None, name=None, admin=True, password=None):
	with db.session() as s:
		domainid = s.get.domainid(config.domain)

		user = DotDict()
		user.handle = Input('Name which will be used to identify yourself on the fediverse') if not handle else handle

		if domainid and s.get.user(handle=handle, domain=config.domain):
			return f'Error: User already exists: {handle}@{config.domain}'

		user.password = Input('Password', password=True) if not password else password

		if user.password != Input('Password again', password=True):
			logging.error('Passwords don\'t match')
			return

		user.email = Input('E-Mail address') if not email else email
		user.name = Input('Display name', user.handle) if not email else name
		user.admin = Input('Make user an admin?', True, Boolean)

		s.put.user(user.handle,
			name = user.name,
			password = mkhash(user.password),
			email = user.email,
			permissions = 1 if Boolean(user.admin) else 4
		)

	return f'Created new user: {user.handle}'


def man_deluser(handle=None, domain=None):
	if not handle:
		return 'Error: Please specify a user handle to delete'

	with db.session() as s:
		if not domain:
			domain = config.domain
			domainid = s.get.domainid(config.domain)

		if not domainid:
			return f'Error: Cannot find domain: {domain}'

		row = s.fetch('user', handle=handle, domainid=domainid)

		if row:
			row.delete_session(s)
			return f'Deleted user: {handle}@{domain}'

		return f'Error: User does not exist: {handle}@{domain}'


def man_migrate():
	config_version = db.get.version()

	if config_version < config.dbversion:
		migrate.upgrade(config_version, db)


def man_getemoji(shortcode, domain=None):
	with db.session() as s:
		row = s.get.emoji(shortcode, domain)

	if not row:
		return f'Error: Cannot find {shortcode}'

	string = str(row)

	return string[:-1] + f', path="{row.path}")'


def man_importemoji(filename=None):
	if not filename:
		return 'Error: Please specify an archive to import'

	success = 0
	fail = 0
	exist = 0
	m = magic.Magic(mime=True, uncompress=True)
	mime = m.from_file(filename)

	if not mime.endswith('x-tar'):
		return f'Error: Unsupported archive mimetype: {mime}'

	with tarfile.open(filename) as a:
		em = a.getmembers()[0]

		with db.session() as s:
			for emoji in a.getmembers():
				if emoji.isfile() and emoji.name.endswith('png') and emoji.path == emoji.name:
					if emoji.size > s.get.config('emoji_size_limit'):
						logging.warning(f'Emoji too big: {emoji.name}')
						fail += 1
						continue

					#[use regex to make sure emojis are only alphanumeric with underscores here]

					shortcode = emoji.name.split('.')[0]
					domain = s.fetch('domain', domain=config.domain)
					path = config.data.join(f'media/emojis/{domain.id}/{emoji.name[0]}')
					emfile = path.join(emoji.name)
					exists = s.fetch('emoji', shortcode=shortcode, domainid=domainid)
					relpath = path.str().replace(config.data.str() + '/', '')
					path.mkdir()

					if not exists:
						s.insert('emoji',
							shortcode = shortcode,
							domainid = domain.id,
							display = True,
							enabled = True,
							timestamp = datetime.now()
						)

						exists = True

					if exists and not emfile.exists():
						#logging.info(f'Extracting {emoji.name} to {emfile}')
						a.extract(emoji, path=path.str(), set_attrs=False)
						success += 1

					else:
						exist += 1

		ret = f'Imported emojis: {success}'

		if exist:
			ret += f'\nExisting emojis not imported: {exist}'

		if fail:
			ret += f'\nFailed imports: {fail}'

		return ret


def man_config(key=None, value=None):
	with db.session() as s:
		if not key:
			for k,v in s.get.configs().items():
				print(f'{k}: {v}')

			return ''

		if not value:
			return f'{key}: {s.get.config(key)}'

		if s.put.config(key, value):
			return f'Updated config: {key}'

		else:
			return f'Failed to update config: {key}'


# keeping this here for reference when postgresql support is added
def man_setup2():
	database = DB.database

	with trans(dbconn('postgres', pooled=False)) as db:
		if '--dropdb' in sys.argv:
			db.prepare('terminate', "SELECT pg_terminate_backend(pid) FROM pg_stat_activity WHERE datname = '$1';")
			db.prepare('drop', 'DROP DATABASE $1')
			db.prepare('create', 'CREATE DATABASE $1 WITH TEMPLATE = template0;')

			db.query_prepared('terminate', [database])
			db.query_prepared('drop', [database])

		if database not in db.get_databases():
			logging.info('Database doesn\'t exist. Creating it now...')
			db.query_prepared('create', [database])

	with trans(dbconn(database, pooled=False)) as db:
			dbsql = open(script_path+'/dist/database.sql').read().replace('\t', '').replace('\n', '')
			db.query(dbsql)

			dbcheck = db.query('SELECT * FROM settings WHERE setting = \'setup\'').dictresult()

			if dbcheck == []:
				keys = genkey()

				settings = {
					'setup': True,
					'pubkey': keys['pubkey'],
					'privkey': keys['privkey'],
					'char_limit': 4096,
					'table_limit': 8,
					'name': 'Barkshark Social',
					'description': 'UwU',
					'theme': 'blue',
					'domain': config['domain']
				}

				for key in settings:
					db.insert('settings', setting=key, val=settings[key])

				logging.info('Database setup finished :3')


# I forget why this even exists, but I'll keep it here for now
def man_convertfonts():
	font_path = script_path + '/static/fonts'

	woffpath = Try(getBin, 'woff2_compress')

	if not woffpath.result:
		logging.error(woffpath.exception, 'Maybe run "sudo apt install woff2"?')
		return

	for directory in os.listdir(font_path):
		files = os.scandir(f'{font_path}/{directory}')

		for font in files:
			name, ext = os.path.splitext(font.path)
			newfont = name + '.woff2'

			if ext in ['.otf', '.ttf'] and not os.path.isfile(newfont):
				cmd = f'{woffpath.result} {font.path}'
				proc = Popen(cmd.split())


def setup_env_file():
	data = f'''# General stuff
WEB_DOMAIN={config.web_domain}
LISTEN={config.listen}
PORT={config.port}
PASS_SALT={config.salt}
LOG_LEVEL=info
LOG_ERRORS=yes

# Federation stuff
DOMAIN={config.domain}

# Random string used to verify proxy headers
FORWARDED_SECRET={config.secret}

DB_TYPE={config.dbtype}
SQ_DATABASE={config.sqfile}

# Postgresql stuff
DB_HOST={config.db.host}
DB_PORT={config.db.port}
DB_USER={config.db.user}
DB_PASSWORD={config.db.password}
DB_DATABASE={config.db.database}
DB_CONNECTIONS={config.db.maxconnections}

# Redis stuff
REDIS_HOST={config.rd.host}
REDIS_PORT={config.rd.port}
REDIS_USER={config.rd.user}
REDIS_PASSWORD={config.rd.password}
REDIS_DATABASE={config.rd.database}
REDIS_PREFIX={config.rd.prefix}
REDIS_CONNECTIONS={config.rd.maxconnections}

# Http client proxy stuff
PROXY_ENABLED={config.proxy.enabled}
PROXY_TYPE={config.proxy.ptype}
PROXY_HOST={config.proxy.host}
PROXY_PORT={config.proxy.port}
'''

	with Path(config.data).join(config.env+'.env').open('w') as fd:
		fd.write(data)

	logging.info(f'Environment file "{config.env}.env" saved')


if __name__ == '__main__':
	cmd = globals().get('man_'+command)

	if not cmd:
		logging.error('Invalid command:', command)
		sys.exit()

	print(cmd(*args))
