from IzzyLib.misc import ApDate, DotDict
from uuid import uuid4

from .config import config
from .functions import ParseText
from .database import db


###
# Messages
###
def Note(status):
	status = db.fetch('status', id=status) if type(status) in [int, str] else status
	user = status.user
	date = ApDate(status.timestamp)
	short_date = date.split('T', 1)[0]

	actor_url = f'https://{config.web_domain}/actor/{user.handle}'
	object_url = f'https://{config.web_domain}/status/{status.id}'
	content = ParseText(status.content)

	data = DotDict({
		'@context': [
			'https://www.w3.org/ns/activitystreams',
			{
				'ostatus': 'http://ostatus.org#',
				'atomUri': 'ostatus:atomUri',
				'inReplyToAtomUri': 'ostatus:inReplyToAtomUri',
				'conversation': 'ostatus:conversation',
				'sensitive': 'as:sensitive',
				'toot': 'http://joinmastodon.org/ns#',
				'votersCount': 'toot:votersCount'
			}
		],
		'id': f'{object_url}',
		'type': 'Note',
		'summary': status.warning,
		'inReplyTo': None,
		'published': date,
		'url': f'https://{config.web_domain}/:{status.id}',
		'attributedTo': actor_url,
		'sensitive': True if status.warning else False,
		'atomUri': object_url,
		'inReplyToAtomUri': None,
		'conversation': f'tag:{user.domain},{short_date}:objectId={status.id}:objectType=Conversation',
		'content': content,
		'contentMap': {
			'en': content
		},
		'attachment': [],
		'tag': [],
		#'replies': {
			#'id': f'{object_url}/replies',
			#'type': 'Collection',
			#'first': {
				#'type': 'CollectionPage',
				#'next': f'{object_url}/replies?min_id=1&page=true',
				#'partOf': f'{object_url}/replies',
				#'items': [
					#object_url
				#]
			#}
		#}
	})

	return set_visibility(data, status, user)


def Actor(request, user):
	user = db.fetch('user', handle=user) if type(user) == str else user

	if not user:
		logging.verbose('User not found when generating actor')
		raise NotFound('User not found')

	sysaccounts = ['system', 'relay', 'local']

	handle = user.handle
	display = user.name
	bio = user.bio
	pubkey = user.pubkey

	actor_url = f'https://{config.web_domain}/actor/{handle}'

	data = DotDict({
		'@context': [
			'https://www.w3.org/ns/activitystreams',
			'https://w3id.org/security/v1',
			{
				'manuallyApprovesFollowers': 'as:manuallyApprovesFollowers',
				'featured': {
					'@id': 'toot:featured',
					'@type': '@id'
				},
				'alsoKnownAs': {
					'@id': 'as:alsoKnownAs',
					'@type': '@id'
				},
				'movedTo': {
					'@id': 'as:movedTo',
					'@type': '@id'
				},
				'schema': 'http://schema.org#',
				'PropertyValue': 'schema:PropertyValue',
				'value': 'schema:value',
				'Emoji': 'toot:Emoji',
			}
		],
		'id': actor_url,
		'type': 'Application' if handle == 'instance' else 'Person',
		'following': f'{actor_url}/following',
		'followers': f'{actor_url}/followers',
		'inbox': f'{actor_url}',
		'featured': f'{actor_url}/featured',
		'preferredUsername': f'{user.handle}',
		'url': f'https://{config.web_domain}/about' if handle in sysaccounts else f'https://{config.web_domain}/@{user.handle}',
		'manuallyApprovesFollowers': False,
		'publicKey': {
			'id': f'{actor_url}#main-key',
			'owner': actor_url,
			'publicKeyPem': f'{user.pubkey}'
		},
		'endpoints': {
			'sharedInbox': f'https://{config.web_domain}/inbox'
		},
	})

	if handle not in sysaccounts and request.ctx.valid:
		data.update({
			'name': f'{user.name}',
			'summary': ParseText(f'{user.bio}'),
			'outbox': f'{actor_url}/outbox',
			'attachment': [],
			'icon': {
				'type': 'Image',
				'mediaType': 'image/png',
				'url': f'https://{config.web_domain}/{user.avatar}'
			},
			'image': {
				'type': 'Image',
				'mediaType': 'image/png',
				'url': f'https://{config.web_domain}/{user.header}'
			}
		})

		for k,v in user.info_table.items():
			data['attachment'].append({
				'type': 'PropertyValue',
				'name': k,
				'value': ParseText(v)
			})

	return data


def Tombstone(objectid):
	data = DotDict({
		'id': objectid,
		'type': 'Tombstone',
		'atomUri': objectid
	})

	return data


###
# Actions
###
#def Create(url, row, user):
	#data = {
		#"@context": [
			#"https://www.w3.org/ns/activitystreams",
			#{
				#"ostatus": "http://ostatus.org#",
				#"atomUri": "ostatus:atomUri",
				#"inReplyToAtomUri": "ostatus:inReplyToAtomUri",
				#"conversation": "ostatus:conversation",
				#"sensitive": "as:sensitive",
				#"toot": "http://joinmastodon.org/ns#",
				#"votersCount": "toot:votersCount",
				#"litepub": "http://litepub.social/ns#",
				#"directMessage": "litepub:directMessage"
			#}
		#],
		#"id": f"{url}/activity",
		#"type": "Create",
		#"actor": f"https://{config.web_domain}/actor/{user.handle}",
		#"published": ApDate(row.timestamp),
		##"to": [
			##"https://barkshark.xyz/users/izalia/followers"
		##],
		##"cc": [
			##"https://www.w3.org/ns/activitystreams#Public"
		##],
		#"object": url
	#}

	#return data
	#return set_visibility(data, row, user)


def Create(object):
	actor = GetObjectActor(object)

	if not actor:
		logging.error('messages.Create: Cannot find actor for object:\n', json.dumps(object.asDict(), indent=4))
		return

	data = {
		'@context': 'https://www.w3.org/ns/activitystreams',
		'id': f'{object.id}/activity',
		'type': 'Create',
		'actor': actor,
		'to': object.get('to', ['https://www.w3.org/ns/activitystreams#Public']),
		'cc': object.get('cc', [f'{actor}/followers']),
		'object': object.asDict(),
	}

	return data


def Delete(object):
	actor = GetObjectActor(object)

	if not actor:
		logging.error('messages.Delete: Cannot find actor for object:\n', json.dumps(object.asDict(), indent=4))
		return

	data = {
		'@context': 'https://www.w3.org/ns/activitystreams',
		'id': f'{object.id}#delete',
		'type': 'Delete',
		'actor': actor,
		'to': object.get('to', ['https://www.w3.org/ns/activitystreams#Public']),
		'cc': object.get('cc', [f'{actor}/followers']),
		'object': Tombstone(objectid).asDict()
	}

	return data


def Announce(object):
	actor = GetObjectActor(object)

	if not actor:
		logging.error('messages.Announce: Cannot find actor for object:\n', json.dumps(object.asDict(), indent=4))
		return

	data = {
		'@context': 'https://www.w3.org/ns/activitystreams',
		'id': f'https://{config.web_domain}/object/{uuid4()}',
		'type': 'Announce',
		'actor': actor,
		'to': object.get('to', ['https://www.w3.org/ns/activitystreams#Public']),
		'cc': object.get('cc', [f'{actor}/followers']),
		'object': object.id
	}

	return data


def Follow(user, dest):
	data = {
		"@context": "https://www.w3.org/ns/activitystreams",
		"id": f"https://{config.web_domain}/object/{uuid4()}",
		'to': dest,
		"type": "Follow",
		"actor": f"https://{config.web_domain}/actor/{user}",
		"object": dest
	}

	return data


def Unfollow():
	data = {
		"@context": "https://www.w3.org/ns/activitystreams",
		"id": "https://barkshark.xyz/users/izalia#follows/354/undo",
		"type": "Undo",
		"actor": "https://barkshark.xyz/users/izalia",
		"object": {
			"id": "https://barkshark.xyz/a3b68a17-1a4e-465e-8460-a1f1c9317566",
			"type": "Follow",
			"actor": "https://barkshark.xyz/users/izalia",
			"object": "https://social.barkshark.xyz/actor/izalia"
		}
	}


###
# Functions
###
def GetObjectActor(object):
	if object.get('type') in ['Person', 'Application']:
		return object['id']

	return object.get('attributedTo', object.get('actor'))


def objecturl(objectid, objtype=None):
	if objectid.startswith(f'https://'):
		return objectid

	if not objtype or objtype not in ['actor', 'status']:
		return

	return f'https://{config.web_domain}/{objtype}/{objectid}'


def get_post_data(objectid):
	post = get.post(objectid) if objectid else postdict

	if not post:
		raise NotFound('That post doesn\'t exist.')

	user = get.user(post['userid'])

	if not user:
		raise NotFound('Some how you found a post without a valid user')

	return (post, user)


def set_visibility(data, status, user):
	visibility = status.visibility
	handle = user.handle
	actor_url = f'https://{config.web_domain}/user/{handle}'

	if status.visibility == 'public':
		data.update({
			'to': ['https://www.w3.org/ns/activitystreams#Public'],
			'cc': [f'{actor_url}/followers']
		})

	elif status.visibility == 'unlisted':
		data.update({
			'to': [f'{actor_url}/followers'],
			'cc': ['https://www.w3.org/ns/activitystreams#Public']
		})

	elif status.visibility == 'private':
		data.update({
			'to': [f'{actor_url}/followers'],
			'cc': []
		})

	return data
