import io

from IzzyLib import logging
from IzzyLib.misc import Boolean, DotDict, DefaultDict
from PIL import Image

from .config import config
from .database import db
from .functions import mkhash, themes


bool_opts = [
	'require_approval',
	'open',
	'secure',
	'private_profiles'
]


def process_media(user, data, mtype):
	filename = mkhash(data, user.get(mtype), 'md5')
	oldpath = config.data.join(user.avatar)
	mediapath = f'media/{mtype}/{user.domain}/{user.handle[0]}/{filename}.png'
	path = config.data.join(mediapath)
	path.parent().mkdir()

	if path.exists():
		path.delete()

	image = Image.open(io.BytesIO(data))
	path.parent().mkdir()
	image.save(path.str())

	user.config[mtype] = filename
	user[mtype] = mediapath

	if oldpath.exists():
		oldpath.delete()


def get_user_profile(request):
	data = {
		'color_themes': list(themes.keys())
	}

	return data


def post_user_profile(request):
	user = request.ctx.cookie_user
	form = DefaultDict(request.ctx.form)
	data = DotDict({k: v for k,v in form.items() if k in user.keys()})
	avatar = request.ctx.files.get('avatar')
	header = request.ctx.files.get('header')

	if avatar and avatar.name:
		process_media(user, avatar.body, 'avatar')

	if header and header.name:
		process_media(user, header.body, 'header')

	if form.oldpassword:
		if not form.password1 or not form.password2:
			return {'error': 'Missing a password field'}

		if form.password1 != form.password2:
			return {'error': 'Passwords don\'t match'}

		curr_hash = mkhash(form.oldpassword)

		if curr_hash != user.password:
			return {'error': 'Current password invalid'}

		data.password = mkhash(form.password1)

	user.config.private = Boolean(form.get('private'))
	user.config.theme = form.get('color_theme')
	print(form.get('color_theme', form))
	user.update(data)

	request.ctx.cookie_user = user

	return {'message': 'Updated profile'}


def get_admin_dashboard(request):
	with db.session() as s:
		domainid = s.get.domainid(config.domain)
		sys_accts = [row.id for row in s.fetch('user', permissions=0, single=False)]
		user = s.table.user
		status = s.table.status

		data = {
			'local_users': s.query(user).filter(user.domainid == domainid, user.permissions > 0).count(),
			'remote_users': s.query(user).filter(user.domainid != domainid).count(),
			'local_statuses': s.query(status).filter(status.domainid == domainid, status.userid.notin_(sys_accts)).count(),
			'remote_statuses': s.query(status).filter(status.domainid != domainid).count(),
		}

	if config.dbtype == 'sqlite':
		size = round(config.sqfile.size()/1024/1024, 2)
		data['dbsize'] = f'{size} MiB'

	else:
		size = 'n/a'

	return data


def get_admin_settings(request):
	with db.session() as s:
		rows = s.fetch('user', domainid=config.domainid, permissions=1, single=False)
		data = {
			'admins': [row.handle for row in rows]
		}

	return data


def post_admin_settings(request):
	user = request.ctx.cookie_user
	form = DefaultDict(request.ctx.form)

	if user and user.permissions != 1:
		return {'error': 'Not authorized'}

	if not form.admin or form.admin == 'none':
		form.admin = ''

	for v in bool_opts:
		if v not in form.keys():
			form[v] = False

	with db.session() as s:
		for k,v in form.items():
			if k in bool_opts:
				s.put.config(k, Boolean(v), 'bool')

			else:
				s.put.config(k, v)

	return {'message': 'Updated settings'}
