import json

from IzzyLib import logging
from sanic import exceptions
from sanic.views import HTTPMethodView
from sanic_openapi import doc

from ..api import oauth, mastodon, misc
from ..config import config
from ..functions import JsonResp, Error


MastodonGet = mastodon.Get()
MastodonPost = mastodon.Post()


class Oauth(HTTPMethodView):
	async def get(self, request, name=None):
		return ApiAction(request, oauth.Get, name)


	async def post(self, request, name=None):
		return ApiAction(request, oauth.Post, name)


class MastodonBase(HTTPMethodView):
	async def get(self, request, path=None):
		return MastodonApiAction(request, MastodonGet, path)


	async def post(self, request, path=None):
		return MastodonApiAction(request, MastodonPost, path)


class Streaming(HTTPMethodView):
	async def get(self, request, name=None):
		async def home(response):
			run = True
			print('Streaming started for', request.ctx.user.handle)

			while run:
				body = await self.request.stream.read()
				data = body.decode()

				await response.write('UvU')

				if data == 'exit':
					run = False

			print('Streaming stopped for', request.ctx.user.handle)

		endpoint = {
			'home': home
		}

		self.request = request
		return stream(self.endpoint.get(name))


def MastodonApiAction(request, method, path):
	if path.endswith('/'):
		path = path[:-1]

	paths = path.split('/')
	base = paths[0]
	name = 'none'
	args = paths[2:] if len(paths) > 2 else []

	if len(paths) > 1:
		name = paths[1]

	if not base:
		return JsonResp('api function not found', 404)

	func = method.get(f'{base}_{name}')

	if not func:
		logging.debug('Mastodon API: Invalid command:', f'{base}_{name}')
		return JsonResp('Invalid command', 404)

	data = PostData(request)
	msg = func(request, data, args)

	logging.warning('Mastodon API:', 'get' if type(method) == MastodonGet else 'post', path, msg)

	return JsonResp(msg) if type(msg) in [dict, list] else msg


def ApiAction(request, modclass, name=None):
	if not name:
		return JsonResp('Missing command', 400)

	data = PostData(request)
	cmd = getattr(modclass, name, False)

	if not cmd:
		return JsonResp('Invalid command', 404)

	msg = cmd(request, data)

	logging.debug('API:', modclass, name, data, msg)

	return JsonResp(msg) if type(msg) in [dict, list] else msg


def PostData(request, name=None):
	try:
		data = request.json

	except exceptions.InvalidUsage:
		data = request.ctx.form

	data = data if data else {}
	data.update(request.ctx.query)

	return misc.sanitize(data, name) if name else data
