from IzzyLib.misc import Path
from PIL import Image
from datetime import datetime
from magic import Magic
from io import BytesIO
from sanic import response
from sanic.views import HTTPMethodView

from ..config import config
from ..database import db
from ..functions import Error, GetTheme


# home page (/)
class Style(HTTPMethodView):
	async def get(self, request, timestamp=None):
		user_config = request.ctx.cookie_user.config if request.ctx.cookie_user else {}
		theme = user_config.get('theme', 'pink')
		path = config.frontend.join('style')

		data = {
			'cssfiles': [css for css in path.listdir() if css.str().endswith('css')]
		}
		data.update(GetTheme(theme))

		return config.template.response('style.css', request, data, ctype='text/css')


class Favicon(HTTPMethodView):
	async def get(self, request):
		filename = config.frontend.join('static/img/icon-64.png').str()
		return await response.file(filename)


class Manifest(HTTPMethodView):
	async def get(self, request):
		with db.session() as s:
			data = {
				'name': s.get.config('name'),
				'short_name': s.get.config('name'),
				'description': s.get.config('description'), #use description_short
				'icons': [
					{
						'src': '/static/img/icon-512.png',
						'sizes': '512x512',
						'type': 'image/png'
					},
					{
						'src': '/static/img/icon-64.png',
						'sizes': '64x64',
						'type': 'image/png'
					}
				],
				'theme_color': '#33bbff',
				'background_color': '#222222',
				'display': 'standalone',
				'start_url': '/',
				'scope': f'https://{config.web_domain}',
				'share_target': {
					'url_template': 'share?title={title}&text={text}&url={url}',
					'action': 'share',
					'method': 'GET',
					'enctype': 'application/x-www-form-urlencoded',
					'params': {
						'title': 'title',
						'text': 'text',
						'url': 'url'
					}
				}
			}


			return response.json(data)


class Media(HTTPMethodView):
	async def get(self, request, path=None):
		path = Path(config.data.join('media').join(path))

		if not path.isfile():
			return Error(request, 404, 'Media not found')

		raw_image = path.open('rb').read()
		width = int(request.ctx.query.get('width', 0))
		height = int(request.ctx.query.get('height', 0))

		if not width and not height:
			try:
				width = height = int(request.query_string)
			except ValueError:
				width = height = None

		if not width and not height:
			mime = Magic(mime=True).from_buffer(raw_image)
			return response.raw(raw_image, content_type=mime)

		if not width:
			width = 2000

		if not height:
			height = 2000

		byte = BytesIO()
		image = Image.open(BytesIO(raw_image))
		image.thumbnail([width, height])
		image.save(byte, format='PNG')

		return response.raw(byte.getvalue(), content_type='image/png')


class Robots(HTTPMethodView):
	async def get(self, request):
		with db.session() as s:
			data = s.get.config('robots')
			return response.text(data)
