import aiohttp
import aiohttp_jinja2
import os
import subprocess
import json
import sys

from os.path import dirname, abspath
from jinja2 import select_autoescape

from social.config import logging, config
from social.functions import color, todate, themes, css_ts
from social import middleware
from social.database import newtrans, get
from social.template_loader import CustomLoader
from .views import main, modules


if getattr(sys, 'frozen', False):
	script_path = dirname(abspath(sys.executable))

else:
	script_path = dirname(abspath(__file__))


async def glob_vars(request):
	return {
		'name': settings['name'],
		'domain': config['web_domain'],
		'settings': get.settings(),
		'get_cookie': get.login_cookie,
		'get_user': get.user,
		'newtrans': newtrans,
		'css_ts': css_ts,
		'lighten': color().lighten,
		'darken': color().darken,
		'saturate': color().saturate,
		'desaturate': color().desaturate,
		'rgba': color().rgba,
		'todate': todate,
		'themes': themes,
		'json': json
	}


webapp = aiohttp.web.Application(middlewares=[])
webapp.on_response_prepare.append(middleware.http_file_cache)

aiohttp_jinja2.setup(webapp,
	loader=CustomLoader(f'{script_path}/templates'),
	autoescape=select_autoescape(['html', 'xml', 'css']),
	context_processors=[glob_vars],
	lstrip_blocks=True,
	trim_blocks=True
)


webapp.router.add_get('', main)
webapp.router.add_static('/js', path=f'{script_path}/js')


def precompile():
	logging.info('Compiling JS')
	subprocess.Popen(['transcrypt', '--map', '--build', '--nomin', '--parent', 'window', f'{script_path}/webapp/js/__init__.py'], preexec_fn=os.setpgrp)
